package org.springframework.web.servlet.view.json.writer.jsonlib;

import java.beans.PropertyEditor;
import java.util.Date;

import org.springframework.beans.PropertyEditorRegistry;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;
import net.sf.json.processors.JsonValueProcessor;
import net.sf.json.processors.JsonVerifier;
import net.sf.json.util.JSONUtils;

public class PropertyEditorRegistryValueProcessor implements JsonValueProcessor{
	
	private PropertyEditorRegistry propertyEditorRegistry;
		

	public PropertyEditorRegistryValueProcessor(PropertyEditorRegistry propertyEditorRegistry) {
		this.propertyEditorRegistry = propertyEditorRegistry;
	}

	@Override
	public Object processArrayValue(Object value, JsonConfig jsonConfig) {
		PropertyEditor pe = null;
		if(propertyEditorRegistry != null){
			pe = propertyEditorRegistry.findCustomEditor(value.getClass(), null);
		}
		
		if(pe == null && (java.util.Date.class.equals(value.getClass()) || java.sql.Date.class.equals(value.getClass())))
			return value.toString();
		
		if(pe != null){
			pe.setValue(value);
			return pe.getAsText();
		}else if(JsonVerifier.isValidJsonValue(value))
			return value;
		else{
			if(JSONUtils.isObject(value))
				return JSONObject.fromObject(value, jsonConfig);
			else
				return JSONArray.fromObject(value, jsonConfig);
		}
	}

	@Override
	public Object processObjectValue(String key, Object value,
			JsonConfig jsonConfig) {
		PropertyEditor pe = null;
		if(propertyEditorRegistry != null && value != null){
			pe = propertyEditorRegistry.findCustomEditor(value.getClass(), null);
		}
		
		if(pe == null && value != null && (java.util.Date.class.equals(value.getClass()) || java.sql.Date.class.equals(value.getClass())))
			return value.toString();
		
		if(pe != null){
			pe.setValue(value);
			return pe.getAsText();
		}else if(JsonVerifier.isValidJsonValue(value))
			return value;
		else{
			if(JSONUtils.isObject(value))
				return JSONObject.fromObject(value, jsonConfig);
			else
				return JSONArray.fromObject(value, jsonConfig);
		}
	}
	

	public PropertyEditorRegistry getPropertyEditorRegistry() {
		return propertyEditorRegistry;
	}



}
