package org.springframework.web.servlet.view.json.writer.sojo;

import java.io.IOException;
import java.io.Writer;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.validation.BindingResult;
import org.springframework.web.servlet.view.json.JsonStringWriter;
import org.springframework.web.servlet.view.json.JsonWriterConfiguratorTemplate;
import org.springframework.web.servlet.view.json.JsonWriterConfiguratorTemplateRegistry;

public class SojoJsonStringWriter implements JsonStringWriter {
	protected final Log logger = LogFactory.getLog(getClass());

	private boolean convertAllMapValues;
	
	@Override
	public void convertAndWrite(Map model, JsonWriterConfiguratorTemplateRegistry configuratorTemplateRegistry, Writer writer, BindingResult br) throws IOException{
		MapGraphWalker walker = new MapGraphWalker();
		walker.getObjectUtil().setWithSimpleKeyMapper(false);
				
		JsonViewWalkerInterceptor jsonInterceptor = new JsonViewWalkerInterceptor();
		jsonInterceptor.setWithNullValuesInMap(true);
		jsonInterceptor.setConvertAllMapValues(convertAllMapValues);
		if(br != null){
			jsonInterceptor.setPropertyEditorRegistry(br.getPropertyEditorRegistry());
			jsonInterceptor.setObjectName(br.getObjectName());
		}
		walker.addInterceptor(jsonInterceptor);
		walker.walk(model);
		
		if(logger.isDebugEnabled())
			logger.debug(jsonInterceptor.getJsonString());
		
		writer.write(jsonInterceptor.getJsonString());
	}

	public boolean isConvertAllMapValues() {
		return convertAllMapValues;
	}

	public void setConvertAllMapValues(boolean convertAllMapValues) {
		this.convertAllMapValues = convertAllMapValues;
	}
	
	

}
